/*
Hijackulator - ZX Spectrum emulator with support for native-code enhancements
Copyright (C) 2003 Matthew Westcott

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Some portions of this software originate from the JEMU project:
http://jemu.emuunlim.com/
*/

/*
 * SpeccyScreen.java
 *
 * Created on 27 April 2003, 17:52
 */

package org.zxdemo.hijackulator;

import java.awt.Graphics;
import java.awt.Color;
import java.awt.Point;

/**
 * Spectrum screen drawing routines
 * @author  Matthew Westcott
 */
public class SpeccyScreen {
    
    public Graphics g;
    protected Memory mem;
    // public Z80 processor = null;
    public boolean pen = true;
    
    protected static final Color[] brightPalette = {
        Color.black, Color.blue, Color.red, Color.magenta,
        Color.green, Color.cyan, Color.yellow, Color.white
    };
    
    /** Creates a new instance of SpeccyScreen */
    public SpeccyScreen(Graphics g, Memory mem) {
        this.g = g;
        this.mem = mem;
    }
    
    public Point pointOfAddress(int address) {
        int y = 0;
        switch(address & 0xf800) {
            case 0x4800:
                y = 64;
                break;
            case 0x5000:
                y = 128;
        }
        y += (address & 0x00E0) >> 2;
        y += (address & 0x0700) >> 8;
        int x = (address & 0x1F) << 3;
        return new Point(x,y);
    }
    
    public void drawByte(int address, int value) {
        if (address < 22528 && pen) {
            int y = 0;
            switch(address & 0xf800) {
                case 0x4800:
                    y = 64;
                    break;
                case 0x5000:
                    y = 128;
            }
            y += (address & 0x00E0) >> 2;
            y += (address & 0x0700) >> 8;
            int x = (address & 0x1F) << 3;

            int attribute = mem.readByte(22528 + (x >> 3) + ((y & 0xF8) << 2));
            Color ink = brightPalette[attribute & 0x07];
            Color paper = brightPalette[(attribute & 0x38) >> 3];
            for (int bit = 0x80; bit > 0; bit >>= 1) {
                g.setColor((value & bit) != 0 ? ink : paper);
                g.fillRect((x++)<<1, y<<1, 2, 2);
            }
        } else if (pen) {
//            System.out.println("Writing attribute ("+address+","+value+")");
            // attribute cell
            int screenbyte;
            switch(address & 0xff00) {
                case 0x5800:
                    screenbyte = 0x4000 | (address & 0xff);
                    break;
                case 0x5900:
                    screenbyte = 0x4800 | (address & 0xff);
                    break;
                default: // 5a00
                    screenbyte = 0x5000 | (address & 0xff);
                    break;
            }
            for (int i=0; i<8; i++) {
                drawByte(screenbyte, mem.readByte(screenbyte));
                screenbyte += 256;
            }
        }
        
        // System.out.println("screen output at "+this.processor.getPC());
    }
    
}
